#!/bin/sh
#
# Script to install Seattle Firewall
#
#     This program is under GPL [http://www.gnu.org/copyleft/gpl.htm]         
#
#     (c) 2000 - Tom Eastep (teastep@evergo.net)
#
#       Seawall documentation is available at http://seawall.sourceforge.net
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of Version 2 of the GNU General Public License 
#       as published by the Free Software Foundation.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
#
#    Usage:
#
#        If you are running a distribution that has a directory called /etc/rc.d/init.d or one
#        called /etc/init.d or you are running Slackware then simply cd to the directory 
#        containing this script and run it.
#
#            ./install.sh
#
#        If you don't have either of those directories, you will need to determine where the
#        SysVInit scripts are kept on your system and pass the name of that directory.
#
#            ./install.sh /etc/rc.d/scripts
#
#        The default is that the firewall will be started in run levels 2-5 starting at
#        position 15 and stopping at position 90. This is correct RedHat/Mandrake, Debian, 
#        Caldera and Corel.
#
#        If you wish to change that, you can pass -r "<levels startpos stoppos>".
#
#        Example 1: You wish to start your firewall in runlevels 2 and three, start at position
#                   15 and stop at position 90
#
#            ./install.sh -r "23 15 90"
#
#       Example 2: You wish to start your firewall only in run level 3, start at position 5 
#                  and stop at position 95.
#
#            ./install.sh -r "3 5 95" /etc/rc.d/scripts
#
#        For distributions that don't include chkconfig (Slackware, for example), the
#        /etc/rc.d/rc.local file is modified to start the firewall.
#

VERSION=3.1

usage() # $1 = exit status
{
    ME=`basename $0`
    echo "usage: $ME [ -r \"<chkconfig parameters>\" ] [ <init scripts directory> ]"
    echo "       $ME [ -v ]"
    echo "       $ME [ -h ]"
    exit $1
}

run_install()
{
    if ( ! install $* ); then
	echo -e "\nERROR: Failed to install $*"
	exit 1
    fi
}

cant_autostart()
{
    echo -e "\nWARNING: Unable to configure Seattle Firewall to start"
echo    "           automatically at boot"
}

backup_file() # $1 = file to backup
{
    if [ -z "$PREFIX" -a -f $1 -a ! -f ${1}-${VERSION}.bkout ]; then
	if (cp $1 ${1}-${VERSION}.bkout); then
	    echo
	    echo "$1 saved to ${1}-${VERSION}.bkout"
        else
	    exit 1
        fi
    fi
}

modify_rclocal()
{
    if [ -f /etc/rc.d/rc.local ]; then
	if [ -z "`grep seawall /etc/rc.d/rc.local`" ]; then
	    cp -f /etc/rc.d/rc.local /etc/rc.d/rc.local-seawall.bkout
	    echo >> /etc/rc.d/rc.local
	    echo "/sbin/seawall start" >> /etc/rc.d/rc.local
	    echo "/etc/rc.d/rc.local modified to start Seattle Firewall"
	fi
    else
	cant_autostart
    fi	
}

install_file_with_backup() # $1 = source $2 = target
{    
    backup_file $2
    run_install -o root -g root -m 0544 $1 ${2}
}

#
# Parse the run line
#
# DEST is the SysVInit script directory
# RUNLEVELS is the chkconfig parmeters for firewall
# ARGS is "yes" if we've already parsed an argument
#
DEST=""
RUNLEVELS=""
ARGS=""

while [ $# -gt 0 ] ; do
    case "$1" in
	-h|help|?)
	    if [ -n "$ARGS" ]; then
		usage 1
            fi

	    usage 0
	    ;;
	-r)
	    if [ -n "$RUNLEVELS" -o $# -eq 1 ]; then
		usage 1
	    fi

	    RUNLEVELS="$2";
	    shift
	    ;;
        -v)
	    if [ -n "$ARGS" ]; then
		usage 1
            fi

	    echo "Seattle Firewall Installer Version $VERSION"
	    exit 0
	    ;;
	*)
	    if [ -n "$DEST" ]; then
		usage 1
            fi

	    DEST="$1"
	    ;;
    esac
    shift
    ARGS="yes"
done
    
#
# Determine where to install the firewall script
#
if [ -n "$PREFIX" ]; then
	install -d -o root -g root -m 755 ${PREFIX}/sbin
	install -d -o root -g root -m 755 ${PREFIX}/etc/rc.d/init.d
fi

FIREWALL="firewall"

if [ -z "$DEST" ]; then
    #
    # We make this first test so that on RedHat systems that have Seawall installed,
    # we can still use PREFIX (the code that reads the existing symbolic link
    # fails dreadfully if the link is relative and PREFIX is non-null).
    #
    if [ -x /etc/rc.d/init.d/firewall ]; then
	DEST=/etc/rc.d/init.d
    elif [ -L /etc/seawall/firewall ]; then
	TEMP=`ls -l /etc/seawall/firewall | sed 's/^.*> //'`
	DEST=`dirname $TEMP`
	FIREWALL=`basename $TEMP`
    elif [ -d /etc/rc.d/init.d ]; then
	DEST=/etc/rc.d/init.d
    elif [ -d /etc/init.d ]; then
	DEST=/etc/init.d
    elif [ -f /etc/rc.d/rc.local ]; then
	DEST=/etc/rc.d
	FIREWALL="rc.firewall"
    else
	echo "ERROR: Can't determine where to install the firewall script"
	echo "       Rerun $0 passing the name of the SysVInit script directory"
	echo "       on your system"
	exit 1
    fi
fi

#
# Change to the directory containing this script
#
cd "`dirname $0`"
    
echo "Installing Seattle Firewall Version $VERSION"

#
# Check for seawall.conf
#
if [ -f ${PREFIX}/etc/seawall.conf ]; then
    first_install=""
else
    first_install="Yes"
fi

#
# Install the control program
#

if [ -z "$PREFIX" -a -z "$first_install" -a ! -f /etc/seawall/version ]; then
    #
    # Could be a 2.x installation
    #
    if [ -f /sbin/seawall ]; then
	oldversion="`grep 'version=' /sbin/seawall | sed 's/version=//'`"
    elif [ -f /usr/sbin/seawall ]; then
	#
	# Get the currently-installed version and move the control program
	# to the standard location
	#
	oldversion="`grep 'version=' /usr/sbin/seawall | sed 's/version=//'`"
	if ( mv /usr/sbin/seawall /sbin ); then
	    echo -e "\n/usr/sbin/seawall moved to /sbin/seawall"
        else
	    echo -e "\nERROR: Please move /usr/sbin/seawall to /sbin/seawall and"
	    echo    "       rerun the install script"
	fi
    else
	echo -e "\nERROR: You have an /etc/seawall.conf file but the installer"
	echo    "       cannot find the seawall program. Either remove /etc/seawall.conf"
	echo    "       or place the seawall program in /sbin/seawall"
	exit 1
    fi
fi
#
# Install /etc/seawall.conf
#
if [ -z "$first_install" ]; then
    backup_file /etc/seawall.conf
    run_install -o root -g root -m 0744 seawall.conf ${PREFIX}/etc/seawall.conf.$VERSION
    echo -e "\nNew configuration file installed as ${PREFIX}/etc/seawall.conf.$VERSION"
    echo    "(Your existing ${PREFIX}/etc/seawall.conf has not been overwritten)"
else
    run_install -o root -g root -m 0744 seawall.conf ${PREFIX}/etc
    echo -e "\nConfiguration file installed as ${PREFIX}/etc/seawall.conf"
fi

install_file_with_backup seawall ${PREFIX}/sbin/seawall

echo -e "\nSeattle Firewall control program installed in ${PREFIX}/sbin/seawall"

#
# Install the Firewall Script
#
if [ -n "$RUNLEVELS" ]; then
    #
    # User specified chkconfig parameters -- build an awk script to install them
    # in the firewall script
    #
    echo "/# chkconfig/ { print \"# chkconfig: $RUNLEVELS\" ; next }" > awk.temp
    echo "{ print }" >> awk.temp

    awk -f awk.temp firewall > firewall.temp

    if [ $? -ne 0 ]; then
	echo -e "\nERROR: Error running awk."
	echo    "         You must run `basename $0` without the "-r" option then edit"
	echo    "         $DEST/$FIREWALL  manually (line beginning '# chkconfig:')"
	exit 1
    fi

    install_file_with_backup firewall.temp ${PREFIX}${DEST}/$FIREWALL
    
    rm -f firewall.temp awk.tmp
else
    install_file_with_backup firewall ${PREFIX}${DEST}/$FIREWALL
fi
    
echo -e "\nSeattle Firewall script installed in ${PREFIX}${DEST}/$FIREWALL"

#
# Create /etc/seawall if needed
#
if [ ! -d ${PREFIX}/etc/seawall ]; then
    mkdir ${PREFIX}/etc/seawall
fi
#
# Install the apps file
#
if [ -f ${PREFIX}/etc/seawall/apps ]; then
    backup_file /etc/seawall/apps
else
    run_install -o root -g root -m 0744 apps ${PREFIX}/etc/seawall/apps
    echo -e "\nApps file installed as ${PREFIX}/etc/seawall/apps"
fi
#
# Install the servers file
#
if [ -f ${PREFIX}/etc/seawall/servers ]; then
    backup_file /etc/seawall/servers
else
    run_install -o root -g root -m 0744 servers ${PREFIX}/etc/seawall/servers
    echo -e "\nServers file installed as ${PREFIX}/etc/seawall/servers"
fi
#
# Install the tunnels file
#
if [ -f ${PREFIX}/etc/seawall/tunnels ]; then
    backup_file /etc/seawall/servers
else
    run_install -o root -g root -m 0744 tunnels ${PREFIX}/etc/seawall/tunnels
    echo -e "\nTunnels file installed as ${PREFIX}/etc/seawall/tunnels"
fi
#
# Backup the version file
#
if [ -z "$PREFIX" ]; then
    if [ -f /etc/seawall/version ]; then
	backup_file /etc/seawall/version
    elif [ -n "$oldversion" ]; then
	echo $oldversion > /etc/seawall/version-${VERSION}.bkout
    else
	echo "Unknown" > /etc/seawall/version-${VERSION}.bkout
    fi
fi
#
# Create the version file
#
echo "$VERSION" > ${PREFIX}/etc/seawall/version
#
# Remove and create the symbolic link to the firewall script
#

if [ -z "$PREFIX" ]; then
    rm -f /etc/seawall/firewall
    ln -s ${DEST}/${FIREWALL} /etc/seawall/firewall
else
    pushd ${PREFIX}/etc/seawall/ >> /dev/null && ln -s ../..${DEST}/${FIREWALL} . && popd >> /dev/null
fi

echo -e "\n${PREFIX}/etc/seawall/firewall linked to ${PREFIX}$DEST/$FIREWALL"

if [ -z "$PREFIX" -a -n "$first_install" ]; then
    if [ -x /sbin/chkconfig -o -x /usr/sbin/chkconfig ]; then
	if ( chkconfig --add $FIREWALL ); then
	    echo -e "\nFirewall will automatically start in run levels as follows:"
	    chkconfig --list $FIREWALL
	else
	    cant_autostart
	fi
    else
       modify_rclocal
    fi
fi
#
#  Report Success
#
echo -e "\nSeattle Firewall Version $VERSION Installed"
